/*
 * Copyright (C) 2005, 2006, 2007 Junjiro Okajima
 *
 * This program, aufs is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* $Id: opts.h,v 1.19 2007/10/15 01:29:18 sfjro Exp $ */

#ifndef __AUFS_OPTS_H__
#define __AUFS_OPTS_H__

#ifdef __KERNEL__

#include <linux/fs.h>
#include <linux/namei.h>
#include <linux/version.h>
#include <linux/aufs_type.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 22)
typedef const char *au_parser_pattern_t;
#else
typedef char *au_parser_pattern_t;
#endif

/* ---------------------------------------------------------------------- */

struct au_opt_add {
	aufs_bindex_t		bindex;
	char			*path;
	int			perm;
	struct nameidata	nd;
};

struct au_opt_del {
	char		*path;
	struct dentry	*h_root;
};

struct au_opt_mod {
	char		*path;
	int		perm;
	struct dentry	*h_root;
};

struct au_opt_xino {
	char		*path;
	struct file	*file;
};

struct au_opt {
	int type;
	union {
		struct au_opt_xino	xino;
		struct au_opt_add	add;
		struct au_opt_del	del;
		struct au_opt_mod	mod;
		int			dirwh;
		int			rdcache;
		int			deblk;
		int			nhash;
		int			udba;
		int			coo;
	};
};

struct au_opts {
	struct au_opt	*opt;
	int		max_opt;

	unsigned int given;
	struct {
		unsigned int remount:1;

		unsigned int refresh_dir:1;
		unsigned int refresh_nondir:1;
		unsigned int trunc_xib:1;
	};
};

/* ---------------------------------------------------------------------- */

int br_perm_str(char *p, unsigned int len, int brperm);
au_parser_pattern_t udba_str(int udba);
void udba_set(struct super_block *sb, unsigned int flg);
au_parser_pattern_t coo_str(int coo);

void au_opts_free(struct au_opts *opts);
int au_opts_parse(struct super_block *sb, char *str, struct au_opts *opts);
int au_opts_mount(struct super_block *sb, struct au_opts *opts);
int au_opts_remount(struct super_block *sb, struct au_opts *opts);

#endif /* __KERNEL__ */
#endif /* __AUFS_OPTS_H__ */
